/*********************************************
 * vim:sw=8:ts=8:si:et
 * To use the above modeline in vim you must have "set modeline" in your .vimrc
 * Author: Guido Socher and Evan Quirk
 * Copyright: GPL V2
 * See http://www.gnu.org/licenses/gpl.html
 *
 * functions to convert ntp timestamps into time and date
 * adapted specifically to drive nixie clock
 *********************************************/
#include <avr/io.h>
#include <stdio.h>
#include <inttypes.h> 
#include <avr/pgmspace.h>

// EPOCH = Jan 1 1970 00:00:00
#define	EPOCH_YR	1970
//(24L * 60L * 60L)
#define	SECS_DAY	86400UL  
#define	LEAPYEAR(year)	(!((year) % 4) && (((year) % 100) || !((year) % 400)))
#define	YEARSIZE(year)	(LEAPYEAR(year) ? 366 : 365)
uint8_t tm_hourh,tm_hourl,tm_minh,tm_minl,tm_sech,tm_secl,prevhourh,prevhourl,prevminh,prevminl,prevsech,prevsecl;
// isleapyear = 0-1
// month=0-11
// return: how many days a month has
//
// We could do this if ram was no issue:
//uint8_t monthlen(uint8_t isleapyear,uint8_t month){
//const uint8_t mlen[2][12] = {
//		{ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
//		{ 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }
//	};
//	return(mlen[isleapyear][month]);
//}
//
uint8_t monthlen(uint8_t isleapyear,uint8_t month){
	if(month==1){
		return(28+isleapyear);
	}
	if (month>6){
		month--;
	}
	if (month%2==1){
		return(30);
	}
	return(31);
}

// gmtime -- convert calendar time (sec since 1970) into broken down time
// returns something like Fri 2007-10-19 in day and 01:02:21 in clock
// sets each digit of clock to either time or date depending on mode
uint8_t gmtime(const uint32_t time, const uint8_t wantyear, const uint8_t tfhr, char *clock){
	if (wantyear == 1) {
		uint16_t dayno;
		uint8_t tm_mon;
		uint8_t tm_day;
		uint16_t tm_year = EPOCH_YR;
		dayno = time / SECS_DAY;
		tm_day = (dayno + 3) % 7;
		

		while (dayno >= YEARSIZE(tm_year)) {
			dayno -= YEARSIZE(tm_year);
			tm_year++;
		}
		tm_mon = 0;
		while (dayno >= monthlen(LEAPYEAR(tm_year),tm_mon)) {
			dayno -= monthlen(LEAPYEAR(tm_year),tm_mon);
			tm_mon++;
		}
		tm_minh = (dayno+1) / 10;
		tm_minl = (dayno+1) - (tm_minh * 10);
		tm_hourh = (tm_mon+1) / 10;
		tm_hourl = (tm_mon+1) - (tm_hourh * 10);
		tm_sech = (tm_year - ((tm_year / 100) * 100)) / 10;
		tm_secl = (tm_year - ((tm_year / 100) * 100)) - (tm_sech * 10);
        sprintf_P(clock,PSTR("%d%d/%d%d/%d%d:%d"),tm_hourh,tm_hourl,tm_minh,tm_minl,tm_sech,tm_secl,tm_day);
        return(tm_day);

	}
        else {
	        uint32_t dayclock;
                if (tfhr == 1){dayclock = time % SECS_DAY;}
                else{dayclock = time % 43200UL;}

	        tm_sech = (dayclock % 60UL) / 10;
	        tm_minh = (dayclock % 3600UL) / 600;
	        tm_hourh = (dayclock / 3600UL) / 10;

	        tm_secl = (dayclock % 60UL) - (tm_sech * 10);
	        tm_minl = ((dayclock % 3600UL) / 60) - (tm_minh * 10);
	        tm_hourl = (dayclock / 3600UL) - (tm_hourh * 10) ;
	        sprintf_P(clock,PSTR("%d%d:%d%d:%d%d"),tm_hourh,tm_hourl,tm_minh,tm_minl,tm_sech,tm_secl);
                return((dayclock % 3600UL) / 60);
        }
}

